
#ifndef RUUVI_H
#define RUUVI_H
#include <iostream>
#include <sstream>
#include <cstdio>
#include <fstream>
#include <sstream>
#include <streambuf>
#include <string>
#include <cstring>
#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <map>
#include <vector>
#include <set>

// Ruuvitag measurement item
struct ruuviMeasurement
{
std::string szBeaconId;
int iDataFormat;
float flHumidity;
float flTemperature;
float flBattery;
unsigned long  ulPressure;
short int siAccelx;
short int siAccely;
short int siAccelz;
};

// Parameter lengths and  ofsets from the manufacturer id.
const int ioBeaconId = -33;
const int ilBeaconId = 17;
const int ioDataFormat = 9;
const int ilDataFormat = 2;
const int ioHumidity = 12;
const int ilHumidity = 2;
const int ioHumidity5 = 18;
const int ilHumidity5 = 6;
const int ioTemp5 = 12;
const int ilTemp5 = 5;
const int ioTemp1 = 15; // Sign and decimal part of temperature.
const int ilTemp1 = 2;
const int ioTemp2 = 18;  // 1/100th part of temperature.
const int ilTemp2 = 2;
const int ioPressure = 21;
const int ilPressure = 6;
const int ioPressure5 = 24;
const int ilPressure5 = 5;
const int ioAccelx = 27;
const int ilAccelx = 5;
const int ioAccely = 33;
const int ilAccely = 5;
const int ioAccelz = 39;
const int ilAccelz = 5;



const int ioAccelx5 = 33;
const int ilAccelx5 = 5;
const int ioAccely5 = 39;
const int ilAccely5 = 5;
const int ioAccelz5 = 45;
const int ilAccelz5 = 5;

const int ioBattery = 44;
const int ilBattery = 6;
const int ioBattery5 = 48;
const int ilBattery5 = 5;



class Ruuvi
{
public:
Ruuvi(const std::string& rszTagId,
      const std::string& rszBluetoothDevice,
      const std::string& rszTargetParams);
~Ruuvi(void);
static void RemoveFile(const std::string& rszFilename);
static void ReleaseResources(void);
static size_t GetFileSize(const std::string& rszFilename);
static size_t RegExprMatchpoint(const std::string& rszString, const std::string& rszRe); 
static void RemoveLinebreaks( std::string& szStr);
static void ReplaceSubstring( std::string& szStr, const std::string& rszSubstr, const std::string& rszReplacestr);
static void PrintValidParams(void);
static void Tolowercase(std::string& rStr);
static const char* HexCharToBin(char c);
static std::string HexStrToBinStr(const std::string& strHex);
std::string FormBeaconId(const std::string& rszHciDumpText);
std::string GetRuuviTagId(void);
void  SetRuuviResult(std::string& rszResult);
std::string GetRuuviResult(void);
void PrintBroadcastData(void);
static bool AreParamsSupported( std::string& szParams);
static void Scan_Dump(const std::string& rszBluetoothDev, const std::string& rszResultFile);
ruuviMeasurement GetRuuviMeasurement(const std::string& rszBroadcastData);
bool RuuviMeasurementExists (const ruuviMeasurement& stRm);
private:
std::string mszRuuviResult; // Detected broadcast as a single string.
std::vector<std::string> mvbroadcastData; // Detected broadcast split Split the result into substrings each representing a single broadcast data element. Indexing the same as in mvruuviMeasurements.
std::vector<ruuviMeasurement> mvruuviMeasurements; // Detected ruuvi measurement data. Indexing the same as in mvbroadcastData.
std::string mszTagId;
std::string mszTargetParams;
std::string mszBluetoothDevice;
};

#endif
